import base64
import hashlib
import json
import socket
import struct
from typing import Optional
from Crypto.Cipher import AES

from openapi.enums import IntegerChoices
from openapi.providers.base import BaseClient, BaseResult, Token


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 200, '成功'
    INVALID_KEY = 98710201, 'APPID 或 SECRET_KEY 无效'


class Result(BaseResult):
    msg: Optional[str] = ''


class Client(BaseClient):
    NAME = '易知课堂'
    API_VERSION = ''
    API_BASE_URL = 'https://app-api.yizhiweixin.com/open-api'

    def __init__(
        self, app_id, secret,
        decrypt_key=None, decrypt_token=None
    ):
        super().__init__()
        self.codes = Code
        self.app_id = app_id
        self.secret = secret

        self.decrypt_key = decrypt_key
        self.decrypt_token = decrypt_token

    def request(
        self, method, endpoint, params=None, data=None,
        token_request=False
    ) -> Result:
        if not token_request:
            params = params or {}
            if 'access_token' not in params:
                params['access_token'] = self.access_token

        request_url = f'{self.API_BASE_URL}{endpoint}'
        response = self._request(method, request_url, params=params, json=data)
        return Result(**response.json()) if response else Result(code=self.codes.FAIL)

    def fetch_access_token(self):
        result = self.request(
            'get', '/accessToken', params={
                'appid': self.app_id,
                'secret_key': self.secret,
                'grant_type': 'client_credential'
            },
            token_request=True
        )
        if result.code == self.codes.SUCCESS:
            self._token = Token(**result.data)

    def decrypt(self, encrypt, timestamp, nonce, signature):
        data = [self.decrypt_token, timestamp, nonce, encrypt]
        data.sort()

        sign_aligo = hashlib.sha1()
        sign_aligo.update(''.join(data).encode())
        if signature != sign_aligo.hexdigest():
            return Result(code=self.codes.FAIL, msg='签名不一致')

        key = base64.b64decode(f'{self.decrypt_key}=')
        crypter = AES.new(key, AES.MODE_CBC, key[:16])

        plain_text = crypter.decrypt(base64.b64decode(encrypt))
        pad = ord(plain_text[-1:])
        content = plain_text[16:-pad]
        json_length = socket.ntohl((struct.unpack('I', content[:4]))[0])
        json_content = content[4:json_length + 4]
        return Result(
            code=self.codes.SUCCESS, msg='OK',
            data=json.loads(json_content.decode())
        )



import pprint
from openapi.providers.crm.tanmarket import Client

from examples.config import config

tan_market_config = config['tanmarket']
filepath = tan_market_config.pop('filepath')
client = Client(**tan_market_config)
client.add_webhook(config['openapi_webhook'])

pprint.pprint(client.request('post', '/v3/profile-fields', json={}).data)

# /v3/profile-fields/map
FIELDS = [
    {
        'fieldId': '104220',
        'alias': '姓名'
    },
    {
        'fieldId': '105086',
        'alias': '下单时间'
    },
    {
        'fieldId': '104217',
        'alias': '电话'
    },
    {
        'fieldId': '104223',
        'alias': '性别'
    },
    {
        'fieldId': '105475',
        'alias': '数分训练营'
    },
    {
        'fieldId': '105476',
        'alias': 'Py训练营'
    },
    {
        'fieldId': '105091',
        'alias': '推广人'
    },
    {
        'fieldId': '105111',
        'alias': '微信号'
    },
    {
        'fieldId': '105158',
        'alias': '渠道来源'
    },
    {
        'fieldId': '105739',
        'alias': '首次报名课程'
    },
    {
        'fieldId': '104219',
        'alias': '描述'
    },
    {
        'fieldId': '106074',
        'alias': 'UID'
    },
    {
        'fieldId': '109145',
        'alias': '已购课程'
    },
    {
        'fieldId': '109146',
        'alias': '购课总金额'
    },
    {
        'fieldId': '105111',
        'alias': '微信'
    },
    {
        'fieldId': '114038',
        'alias': '新Py训练营'
    },
    {
        'fieldId': '128220',
        'alias': 'Linux训练营'
    }
]

if __name__ == '__main__':
    result = client.request('post', '/v3/profile-fields/map', json=FIELDS[-1])
    print(result)
    # for alias in FIELDS[-2:]:
    #     print(alias)
    #     result = client.request('post', '/v3/profile-fields/map', json=alias)
    #     print(alias, result)
    # result = client.request('post', '/v3/profile-fields', json={})
    # print(result)



import typing

from openapi.providers.base import BaseClient, BaseResult, Token
from openapi.enums import IntegerChoices


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'


class Result(BaseResult):
    msg: typing.Optional[str]


class Client(BaseClient):
    NAME = '飞书 - 开发平台'
    API_BASE_URL = 'https://open.feishu.cn/open-apis'
    API_VERSION = 'v3'

    def __init__(self, app_id, secret, version=API_VERSION):
        super().__init__()
        self.app_id = app_id
        self.secret = secret
        self.version = version

        self.codes = Code

    def fetch_access_token(self):
        result = self.request(
            'post', '/auth/v3/tenant_access_token/internal',
            json={
                'app_id': self.app_id,
                'app_secret': self.secret
            },
            token_request=True
        )
        if result.code == self.codes.SUCCESS:
            self._token = Token(
                access_token=result.data['tenant_access_token'],
                expires_in=result.data['expire']
            )

    def request(
        self, method, endpoint,
        json: typing.Union[typing.Dict, typing.List] = None,
        headers: typing.Optional[typing.Dict] = None,
        token_request=False
    ):
        request_url = f'{self.API_BASE_URL}{endpoint}'
        if not token_request:
            if headers is None:
                headers = {}

            headers['Authorization'] = f'Bearer {self.access_token}'

        response = self._request(method, request_url, headers=headers, json=json)
        if token_request:
            result = response.json()
            return Result(code=result['code'], msg=result['msg'], data=result)
        return Result(**response.json())



import typing

from openapi.providers.base import BaseClient, BaseResult
from openapi.enums import IntegerChoices


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'


class Result(BaseResult):
    StatusCode: typing.Optional[int] = None


class Client(BaseClient):
    NAME = '飞书 - 群组机器人'
    API_BASE_URL = 'https://open.feishu.cn/open-apis/bot/'
    API_VERSION = 'v2'

    def __init__(self, secret=None):
        super().__init__()
        self.secret = secret
        self.codes = Code

    def fetch_access_token(self):
        pass

    def request(
        self, method, endpoint,
        json: typing.Union[typing.Dict, typing.List] = None,
    ):
        request_url = f'{self.API_BASE_URL}{self.API_VERSION}{endpoint}'
        response = self._request(method, request_url, json=json)
        return Result(**(response.json() if response else {'code': self.codes.FAIL}))



from typing import Dict
from openapi.providers.wechat.open import Client, Result

from examples.config import config


class OrderDetailResult(Result):
    order: Dict


d = {
    'ToUserName': 'gh_52dd8ab5ef3c',
    'Encrypt': '/lJ+CuKVDhs2lKp667TAj0hWhvdC+4RbKLcAVS/OoT97+tDATGPShJWE0+h0pQ2Dkl7kX+fQqEa50m7M8ucyMPT3DTjI8zyd4pWQXVqPokn9SDeWATkUQQHBUvyruB9vRsBKkiy+fp44zx6vUd4qoq4/FukF2E+/Vox9dk0G4vT11R/pr1ZwcocYWppzSWjBRv8bSUXF/4VhcaW+mscmBEADWbi8VVU63nV9MRCQK322xsVIFX1ZfOOq9+WRWxcSTDnZepPnqmGzIOfhBelkNNsQAlkDrs3Tr+mHSFR186wwxiYCxQT27fPtk2XII1a5wrZ70uPDfZwFy78g29yuzw=='
}

signature = '80e59b3963a4fc0bdf18428fc52151a778cd0e7e'
timestamp = '1691591499'
nonce = '1355383148'
openid = 'oGKF45TzFbDCkIw53Qi54BzkUKT4'
encrypt_type = 'aes'
msg_signature = 'd96fc6dd703606937b42f639ed8ece9bc29560e6'

client = Client(**config['wechat_channels'])
result = client.decrypt(
    d['Encrypt'],
    timestamp=timestamp, nonce=nonce, signature=msg_signature
)

# client.add_webhook(config['openapi_webhook'])
result = client.request(
    'POST', endpoint='/channels/ec/order/get',
    data={
        'order_id': result.data['order_info']['order_id']
    }, is_oauth=False, result_processor=OrderDetailResult
)
print(result)



import time
import secrets
from pathlib import Path
from typing import Optional, Union, Dict
from base64 import encodebytes, decodebytes
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA256

from openapi.enums import TextChoices
from openapi.exceptions import NotFoundPath
from openapi.providers.base import BaseClient, BaseResult


def format_params(data):
    return '&'.join([f'{k}={data[k]}' for k in sorted(data) if data[k] and k != 'sign'])


def calculate_signature(unsigned_string, api_key):
    signer = PKCS1_v1_5.new(api_key)
    signature = signer.sign(SHA256.new(unsigned_string))
    # base64 编码，转换为unicode表示并移除回车
    return encodebytes(signature).decode('utf-8').replace('\n', '')


def verify_signature(params, api_key, charset='utf-8'):
    sign = params['sign']
    string = format_params(params)
    signer = PKCS1_v1_5.new(api_key)
    digest = SHA256.new()
    digest.update(string.encode(charset))
    return signer.verify(digest, decodebytes(sign.encode(charset)))


class Code(TextChoices):
    SUCCESS = '10000', 'OK'
    SIGN_FAIL = '401', '签名错误'
    FAIL = '500', '错误信息'


class Result(BaseResult):
    code: Union[Code, str] = Code.SUCCESS
    data: Optional[Dict]
    message: Optional[str]


class Client(BaseClient):
    NAME = '联想支付'
    VERSION = '1.0'
    API_BASE_URL = 'https://cloud-rest.lenovomm.com/cloud-intermodal-core/api/v1'

    def __init__(self, app_id, mch_id, private_key_path, public_key_path):
        super().__init__()
        self.app_id = app_id
        self.mch_id = mch_id
        self.codes = Code

        public_key_path = Path(public_key_path)
        if not public_key_path.exists():
            raise NotFoundPath(f'{public_key_path} not found')

        private_key_path = Path(private_key_path)
        if not private_key_path.exists():
            raise NotFoundPath(f'{private_key_path} not found')

        # 加载公钥
        with open(public_key_path) as fp:
            self.public_key = RSA.importKey(fp.read())

        # 加载私钥
        with open(private_key_path) as fp:
            self.private_key = RSA.importKey(fp.read())

    def request(self, endpoint: str, data: Dict, method: str = 'post') -> Result:
        public_params = {
            'mchId': self.mch_id,
            'appId': self.app_id,
            'nonce': secrets.token_hex(16),
            'timestamp': str(int(time.time() * 1000)),
            'version': self.VERSION,
            'signType': 'RSA2'
        }
        data.update(**public_params)

        data['sign'] = calculate_signature(
            format_params(data).encode(),
            self.private_key
        )
        request_url = f'{self.API_BASE_URL}{endpoint}'
        response = self._request(method, request_url, json=data)
        return Result(**response.json())

    def check_signature(self, data: Dict) -> bool:
        data.setdefault('appId', self.app_id)
        data.setdefault('mchId', self.app_id)
        return bool(verify_signature(data, self.public_key))

    def fetch_access_token(self):
        pass



import base64
import hashlib
import json
import socket
import struct
from urllib.parse import quote
from typing import Optional
from Crypto.Cipher import AES

from openapi.providers.base import BaseClient, BaseResult, Token
from openapi.exceptions import DisallowedHost, OpenAPIException
from openapi.enums import IntegerChoices


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'
    INVALID_WHITE_LIST = 40164, 'ip 未在白名单'


class Result(BaseResult):
    errcode: int = Code.SUCCESS
    errmsg: Optional[str]
    msgid: Optional[int]


class Client(BaseClient):
    NAME = '微信服务号/视频号'
    API_BASE_URL = 'https://api.weixin.qq.com/cgi-bin'
    API_VERSION = ''

    def __init__(
        self, app_id, secret,
        decrypt_key=None, decrypt_token=None
    ):
        super().__init__()
        self.app_id = app_id
        self.secret = secret
        self.codes = Code

        self.decrypt_key = decrypt_key
        self.decrypt_token = decrypt_token

    def request(
        self, method, endpoint, params=None, data=None,
        token_request=False, is_oauth=True, replace_url=True,
        result_processor=None
    ):
        if not token_request:
            if params is None:
                params = {}

            if 'access_token' not in params and not is_oauth:
                params['access_token'] = self.access_token

        # Support wechat oauth login
        api_base_url = self.API_BASE_URL
        if not is_oauth and replace_url:
            api_base_url = self.API_BASE_URL.replace('/cgi-bin', '')

        request_url = f'{api_base_url}{endpoint}'
        response = self._request(
            method, request_url,
            params=params, json=data
        )
        if response is None:
            return Result(code=self.codes.FAIL)

        result = response.json()
        if 'errcode' in result:
            return Result(**result) if result_processor is None else result_processor(**result)
        else:
            return Result(data=result)

    def check_token(self, access_token):
        result = self.request(
            'get', '/get_api_domain_ip',
            params={'access_token': access_token}
        )
        return result.code == self.codes.SUCCESS

    def fetch_access_token(self):
        result = self.request(
            'get', '/token', params={
                'grant_type': 'client_credential',
                'appid': self.app_id,
                'secret': self.secret
            },
            token_request=True
        )
        if result.errcode == self.codes.SUCCESS:
            self._token = Token(**result.data)
        else:
            if result.errcode == self.codes.INVALID_WHITE_LIST:
                raise DisallowedHost(result.errmsg)
            else:
                raise OpenAPIException(result.code, result.errmsg)

    def get_authorize_url(
        self, scope='snsapi_base', state='',
        redirect_uri=''
    ):
        redirect_uri = quote(redirect_uri, safe='')
        url_list = [
            'https://open.weixin.qq.com',
            '/connect/oauth2/authorize?appid=',
            self.app_id,
            '&redirect_uri=',
            redirect_uri,
            '&response_type=code&scope=',
            scope
        ]
        if state:
            url_list.extend(['&state=', state])
        url_list.append('#wechat_redirect')
        return ''.join(url_list)

    def get_qrcode_url(self, state='', redirect_uri=''):
        redirect_uri = quote(redirect_uri, safe='')
        url_list = [
            'https://open.weixin.qq.com',
            '/connect/qrconnect?appid=',
            self.app_id,
            '&redirect_uri=',
            redirect_uri,
            '&response_type=code&scope=',
            'snsapi_login'  # scope
        ]
        if state:
            url_list.extend(['&state=', state])
        url_list.append('#wechat_redirect')
        return ''.join(url_list)

    def decrypt(self, encrypt, timestamp, nonce, signature):
        data = [self.decrypt_token, timestamp, nonce, encrypt]
        data.sort()

        sign_aligo = hashlib.sha1()
        sign_aligo.update(''.join(data).encode())
        if signature != sign_aligo.hexdigest():
            return Result(code=self.codes.FAIL, msg='签名不一致')

        key = base64.b64decode(f'{self.decrypt_key}=')
        crypter = AES.new(key, AES.MODE_CBC, key[:16])

        plain_text = crypter.decrypt(base64.b64decode(encrypt))
        pad = ord(plain_text[-1:])
        content = plain_text[16:-pad]
        length = socket.ntohl((struct.unpack('I', content[:4]))[0])
        content = content[4:length + 4]
        try:
            data = json.loads(content.decode())
            return Result(
                code=self.codes.SUCCESS, message='OK',
                data=data
            )
        except Exception as exc:
            return Result(code=self.codes.FAIL, message=str(exc))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2022/12/3
from pydantic import BaseModel

from fastapi import FastAPI

app = FastAPI()


class Message(BaseModel):
    encrypt: str
    msg_signature: str
    timestamp: int
    nonce: str


@app.post(path='/yizhi')
def callback(message: Message):
    print(message)
    return 'success'



from openapi.providers.crm.tanmarket import Client

from examples.config import config

tan_market_config = config['tanmarket']
filepath = tan_market_config.pop('filepath')
client = Client(**tan_market_config)
client.add_webhook(config['openapi_webhook'])

# For testing
result = client.request(
    'post', '/v3/echo',
    json={'hello': 'tanmarket'}
)
# print('>>>>', result)
# code=40417 data=None message='请求参数验证失败：电话号码重复'
# code=0 data=497186786844992 message='success'

if __name__ == '__main__':
    result = client.request('post', '/v3/add-clue', json={
        'customerName': 'liuzhichao-test3',
        'mobiles': ['18803561678'],
        'fields': [
            {
                'alias': '首次报名课程',
                'fieldValue': 'python集训营'
            },
            {
                'alias': '下单时间',
                'fieldValue': '2022-09-14 21:01:00'
            },
            # {
            #     'alias': '电话',
            #     'fieldValue': ['18803561681']
            # },
            {
                'alias': '性别',
                'fieldValue': 0
            },
            {
                'alias': '姓名',
                'fieldValue': '刘志超'
            },
            {
                'alias': '数分训练营',
                'fieldValue': 1
            },
            {
                'alias': '推广人',
                'fieldValue': 'alex'
            },
            {
                'alias': '渠道来源',
                'fieldValue': 0
            },
            {
                'alias': '微信号',
                'fieldValue': '123'
            },
            {
                'alias': '描述',
                'fieldValue': 'hhhh'
            },
            {
                'alias': '已购课程',
                'fieldValue': '测试课程'
            },
            {
                'alias': '购课总金额',
                'fieldValue': 0.12
            },
        ]
    })
    print(result)



import json
from openapi.providers.feishu.bot import Client

from examples.config import config

bot = Client()
# result = bot.request(
#     'post',
#     config['feishu']['bot'],
#     json={
#         'msg_type': 'text',
#         'content': {'text': 'request example'}
#     }
# )
# print(result)
with open('card.json') as f:
    content = json.loads(f.read())['content']
    result = bot.request(
        'post',
        config['feishu']['bot'],
        json={
            'msg_type': 'interactive',
            'card': content
        }
    )
    print(result)



import time
import typing
import hashlib
import json as _json

from openapi.providers.base import BaseClient, BaseResult
from openapi.enums import IntegerChoices


def calc_signature(string: str):
    return hashlib.sha256(string.encode('utf-8')).hexdigest()


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'
    INVALID_SIGN = 1002, '签名校验未通过'
    INVALID_TIMESTAMP = 1003, '请求时间与服务器时间相差过大，请校对时间后重新请求'
    REPEATED_MOBILE = 40417, '请求参数验证失败：电话号码重复'


class Result(BaseResult):
    pass


class Client(BaseClient):
    NAME = '探马'
    API_BASE_URL = 'https://api.tanmarket.cn:20066/api'
    API_VERSION = ''

    def __init__(self, app_id, app_key):
        super().__init__()
        self.app_id = app_id
        self.app_key = app_key

        self.codes = Code

    def fetch_access_token(self):
        pass

    def request(
        self, method, endpoint,
        params: typing.Dict = None,
        data: typing.Union[typing.Dict, typing.List] = None,
        json: typing.Union[typing.Dict, typing.List] = None,
        headers: typing.Dict = None
    ):
        request_url = f'{self.API_BASE_URL}{self.API_VERSION}{endpoint}'
        if headers is None:
            headers = {}

        timestamp = f'{int(time.time() * 1000)}'
        string = f'{self.app_id}{timestamp}{_json.dumps(data or json)}{self.app_key}'
        headers.update(**{
            'appId': self.app_id,
            'timestamp': timestamp,
            'sign': calc_signature(string)
        })
        response = self._request(method, request_url, params, data, json, headers)
        return Result(**(response.json() if response else {'code': self.codes.FAIL}))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2022/6/20
import re
import os
from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as fd:
    long_description = fd.read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('openapi')

setup(
    name='openapipy',
    version=version,
    description='openapi tools',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[],
    keywords='openapi,openapi-python,python-openapi,openapipy,pyopenapi',
    author='ZhiChaoLiu',
    author_email='liuzhichao9527@gmail.com',
    url='https://github.com/luffy-genius/openapi',
    license='MIT',
    packages=find_packages(exclude=('examples',)),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        'httpx',
        'pydantic',
        'pycryptodome',
        'cryptography'
    ],
    entry_points={
      'console_scripts': []
    }
)
